/**
 * Ext GWT 3.0.0-rc - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.desktopapp.client.wordprocessor;

import com.google.gwt.event.dom.client.KeyDownEvent;
import com.google.gwt.event.dom.client.KeyDownHandler;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.util.DelayedTask;
import com.sencha.gxt.desktopapp.client.filemanager.images.Images;
import com.sencha.gxt.widget.core.client.Window;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;
import com.sencha.gxt.widget.core.client.event.ShowEvent;
import com.sencha.gxt.widget.core.client.event.ShowEvent.ShowHandler;
import com.sencha.gxt.widget.core.client.form.HtmlEditor;

public class WordProcessorViewImpl implements WordProcessorView, HideHandler, ShowHandler {

  private static final int AUTO_SAVE_DELAY_MS = 750;
  private static final int AUTO_SAVE_INTERVAL = 10;

  private WordProcessorPresenter wordProcessorPresenter;
  private Window window;
  private HtmlEditor htmlEditor;
  private DelayedTask autoSaveDelayedTask;
  private int autoSaveCount;

  public WordProcessorViewImpl(WordProcessorPresenter wordProcessorPresenter) {
    this.wordProcessorPresenter = wordProcessorPresenter;
  }

  @Override
  public Widget asWidget() {
    return getWindow();
  }

  @Override
  public void close() {
    getWindow().hide();
  }

  @Override
  public String getValue() {
    return getHtmlEditor().getValue();
  }

  @Override
  public void onHide(HideEvent event) {
    getWordProcessorPresenter().unbind();
  }

  @Override
  public void onShow(ShowEvent event) {
    getWordProcessorPresenter().bind();
    getHtmlEditor().focus();
  }

  @Override
  public void setTitle(String title) {
    getWindow().setHeadingText(title);
  }

  @Override
  public void setValue(String value) {
    getHtmlEditor().setValue(value);
  }

  private DelayedTask getAutoSaveDelayedTask() {
    if (autoSaveDelayedTask == null) {
      autoSaveDelayedTask = new DelayedTask() {
        @Override
        public void onExecute() {
          getWordProcessorPresenter().onSave();
          autoSaveDelayedTask.delay(AUTO_SAVE_DELAY_MS);
          autoSaveCount = 0;
        }
      };
    }
    return autoSaveDelayedTask;
  }

  private HtmlEditor getHtmlEditor() {
    if (htmlEditor == null) {
      htmlEditor = new HtmlEditor() {
        {
          textArea.addKeyDownHandler(new KeyDownHandler() {
            @Override
            public void onKeyDown(KeyDownEvent event) {
              if (++autoSaveCount > AUTO_SAVE_INTERVAL) {
                getAutoSaveDelayedTask().delay(0);
              } else {
                getAutoSaveDelayedTask().delay(AUTO_SAVE_DELAY_MS);
              }
            }
          });
        }
      };
    }
    return htmlEditor;
  }

  private Window getWindow() {
    if (window == null) {
      window = new Window() {
        {
          removeFromParentOnHide = false;
        }
      };
      window.getHeader().setIcon(Images.getImageResources().page_white());
      window.setMinimizable(true);
      window.setMaximizable(true);
      window.setPixelSize(500, 400);
      window.setOnEsc(false);
      window.addHideHandler(this);
      window.addShowHandler(this);
      window.add(getHtmlEditor());
      getAutoSaveDelayedTask().delay(AUTO_SAVE_DELAY_MS);
    }
    return window;
  }

  private WordProcessorPresenter getWordProcessorPresenter() {
    return wordProcessorPresenter;
  }

}
